from panda3d.core import loadPrcFileData
from direct.showbase.AppRunnerGlobal import appRunner
if appRunner: #run from binary
    path=appRunner.p3dFilename.getDirname()+'/'#+'/autoconfig.txt'    
else:  #run from python 
    path=''#'autoconfig.txt'
try:    
    f = open(path+'autoconfig.txt', 'r')
    for line in f:        
        if not line.startswith('#'):        
            loadPrcFileData('',line)
except IOError:
    print "No config file, using default"
    
import sys
from direct.showbase.DirectObject import DirectObject
from direct.showbase.InputStateGlobal import inputState
from direct.showbase import ShowBase

from panda3d.core import *
from panda3d.bullet import *

from direct.particles.Particles import Particles
from direct.particles.ParticleEffect import ParticleEffect
from direct.particles.ForceGroup import ForceGroup
from direct.interval.IntervalGlobal import *

from camcon import CameraControler
from vfx import vfx
import random
from gui import GuiManager

class Game(DirectObject):
    def __init__(self):
        #init ShowBase
        base = ShowBase.ShowBase()
        base.setBackgroundColor(0, 0, 0, 1)
        base.setFrameRateMeter(True)  
        base.enableParticles()        
        base.disableMouse()
        render.setShaderAuto()
        
        #set up gui
        self.gui=GuiManager(self)
        
        self.controler=CameraControler(self)
        self.isFlying=True
        self.last_volume=0.0
        self.turbo=False
        self.turbo_power=0
        self.color=0 
        self.last_firewall_time=0
        self.last_generate_time=0
        self.pause=True
        self.data=0
        self.time=30.0
        # Input
        inputState.watchWithModifiers('left', 'a')
        inputState.watchWithModifiers('right', 'd')
        inputState.watchWithModifiers('forward', 'w')
        inputState.watchWithModifiers('back', 's')
        
        self.accept( 'window-event', self.onWindowEvent)
        self.accept( 'escape', self.setPause)
        
        # Task
        taskMgr.add(self.update, 'updateWorld')
        taskMgr.doMethodLater(1.0,self.tick, 'tick')
        # Physics
        self.setup()

      # _____HANDLER_____
      
    def setPause(self):
        if self.pause:
            self.gui.startGame()
        else:
            self.pause=True
            self.gui.logo.show()
            self.gui.menu_frame.show()  
               
    def onWindowEvent(self,window=None):
        if window is not None: # window is none if panda3d is not started 
            self.gui.updateGuiNodes()
              
    def generateFirewallColor(self, time):
        if abs(time-self.last_firewall_time)>10.0:
            r=random.randint(0,3)
            c=[VBase4(.75, 1, 1, 1),VBase4(0.4, 1, 0.4, 1), VBase4(1, 0.5, 1, 1), VBase4(1, 0.1, 0.1, 1)]
            color=c[r]
            self.firewall.setColor(color)
            self.gui.firewall.setColor(color)
            self.gui.firewall.show()
            self.last_firewall_time=time
            mask=BitMask32()
            if r==0:
                #mask.setBit(4) #firewall cyan
                mask.setBit(5) #firewall Green
                mask.setBit(6) #firewall Mangana
                mask.setBit(7) #firewall Red
            if r==1:
                mask.setBit(4) #firewall cyan
                #mask.setBit(5) #firewall Green
                mask.setBit(6) #firewall Mangana
                mask.setBit(7) #firewall Red   
            if r==2:
                mask.setBit(4) #firewall cyan
                mask.setBit(5) #firewall Green
                #mask.setBit(6) #firewall Mangana
                mask.setBit(7) #firewall Red   
            if r==3:
                mask.setBit(4) #firewall cyan
                mask.setBit(5) #firewall Green
                mask.setBit(6) #firewall Mangana
                #mask.setBit(7) #firewall Red    
            self.firewall_bullet.setCollideMask(mask)            
            #print "firewall UP!", mask
            
    def reGenerate(self, time):
        if abs(time-self.last_generate_time)>10.0:
            self.last_generate_time=time    
            self.gui.firewall.hide()
            for obj in self.powerups_vfx:
                if obj:
                    obj.remove_loop()
            for obj in self.powerups_ghost:
                if obj:
                    self.world.removeGhost(obj.node())                
            for obj in self.powerups:
                if obj:
                    obj.removeNode()
            for obj in self.bullet_segments:
                if obj:
                    self.world.removeRigidBody(obj.node()) 
            for obj in self.segments:
                if obj:
                    obj.removeNode() 
                
            self.powerups_vfx=[]    
            self.powerups_function=[]
            self.powerups=[]
            self.powerups_ghost=[]
            self.segments=[]
            self.bullet_segments=[]
            
            self.generate(offset=Vec3(0, 500, 0), start_ang=0, ang=10, count=27)
            self.generate(offset=Vec3(0,0, 0), start_ang=90, ang=-10, count=27)
            
    def setTurbo(self):        
        self.turbo_power+=5
        if self.turbo_power>20:
            self.turbo_power=20
        self.sphere_np.node().setMass(1.5)
        self.turbo=True
        self.electric_vfx.vfx.show()
        if not  self.sfx['pickup_turbo'].status()== 2:
            self.sfx['pickup_turbo'].play()
            self.data+=1
            self.time+=1.0
            self.gui.data['text']="%.0f kB"%self.data
    def setCyan(self):  
        self.tron_part.setColor(.75, 1, 1, 1)
        self.electric_vfx.vfx.setColor(.75, 1, 1, 1)
        self.pLight.setColor(VBase4(.75, 1, 1, 1))
        self.color=0 
        mask=BitMask32()
        mask.setBit(1) #level
        # mask.setBit(2) #triggers
        mask.setBit(3) #powerups
        mask.setBit(4) #firewall cyan
        #mask.setBit(5) #firewall Green
        #mask.setBit(6) #firewall Mangana
        #mask.setBit(7) #firewall Red        
        self.sphere_np.setCollideMask(mask)
        #print mask
        if not  self.sfx['pickup1'].status()== 2:
            self.sfx['pickup1'].play()
            self.data+=5
            self.time+=5.0
            self.gui.data['text']="%.0f kB"%self.data
    def setGreen(self): 
        self.tron_part.setColor(0.4, 1, 0.4, 1)
        self.electric_vfx.vfx.setColor(0.4, 1,0.4, 1)
        self.pLight.setColor(VBase4(0.4, 1, 0.4, 1))
        self.color=1
        mask=BitMask32()
        mask.setBit(1) #level
        # mask.setBit(2) #triggers
        mask.setBit(3) #powerups
        #mask.setBit(4) #firewall cyan
        mask.setBit(5) #firewall Green
        #mask.setBit(6) #firewall Mangana
        #mask.setBit(7) #firewall Red        
        self.sphere_np.setCollideMask(mask)
        #print mask
        if not  self.sfx['pickup2'].status()== 2:
            self.sfx['pickup2'].play()
            self.data+=5
            self.time+=5.0
            self.gui.data['text']="%.0f kB"%self.data
    def setMangana(self): 
        self.tron_part.setColor(1, 0.5, 1, 1)
        self.electric_vfx.vfx.setColor(1, 0.5, 1, 1)
        self.pLight.setColor(VBase4(1, 0.5, 1, 1))
        self.color=2
        mask=BitMask32()
        mask.setBit(1) #level
        # mask.setBit(2) #triggers
        mask.setBit(3) #powerups
        #mask.setBit(4) #firewall cyan
        #mask.setBit(5) #firewall Green
        mask.setBit(6) #firewall Mangana
        #mask.setBit(7) #firewall Red        
        self.sphere_np.setCollideMask(mask)
        #print mask
        if not  self.sfx['pickup3'].status()== 2:
            self.sfx['pickup3'].play()
            self.data+=5
            self.time+=5.0
            self.gui.data['text']="%.0f kB"%self.data
    def setRed(self): 
        self.tron_part.setColor(1, 0.1, 0.1, 1)
        self.electric_vfx.vfx.setColor(1, 0.1, 0.1, 1)
        self.pLight.setColor(VBase4(1, 0.1, 0.1, 1))
        self.color=3
        mask=BitMask32()
        mask.setBit(1) #level
        # mask.setBit(2) #triggers
        mask.setBit(3) #powerups
        #mask.setBit(4) #firewall cyan
        #mask.setBit(5) #firewall Green
        #mask.setBit(6) #firewall Mangana
        mask.setBit(7) #firewall Red        
        self.sphere_np.setCollideMask(mask)
        #print mask
        if not  self.sfx['pickup4'].status()== 2:
            self.sfx['pickup4'].play()
            self.data+=5
            self.time+=5.0
            self.gui.data['text']="%.0f kB"%self.data
            
    def setPickup(self):
        if not  self.sfx['pickup5'].status()== 2:
            self.sfx['pickup5'].play()
            self.data+=10
            self.time+=5.0
            self.gui.data['text']="%.0f kB"%self.data
                
    def exit(self):
        sys.exit(1)
        
      # ____TASK___
    def tick(self, task):
        if not self.pause:
            self.data+=1
            self.turbo_power-=1
            self.gui.timer['text']="Time: %.0f"%self.time
            self.gui.data['text']="%.0f kB"%self.data
            if self.turbo_power<1:
                self.turbo_power=0
                self.sphere_np.node().setMass(2.0)
                self.turbo=False
                self.electric_vfx.vfx.hide()
            #print self.turbo_power  
            if  self.time<1:
                self.gui.doGameOver()
                self.pause=True
                self.accept( 'escape', self.exit)
        return task.again
        
    def processInput(self, dt):
        if not self.pause:
            force = Vec3(0, 0, -10.0)
            torque= Vec3(0, 0, 0.0)
            if inputState.isSet('left'):
                torque.setY(-150.0)
                force.setX(-50.0)
            if inputState.isSet('right'):
                torque.setY( 150.0)
                force.setX(50.0)
            if inputState.isSet('forward'):
                force.setY(50.0)
                torque.setX(-150.0)
            if inputState.isSet('back'):
                force.setY(-50.0) 
                torque.setX(150.0)        

            if self.isFlying:
                if not self.turbo:            
                    force*=0.8
                force*=0.8  
                torque*=0.1            
            torque = render.getRelativeVector(self.controler.cameraNode, torque)
            force = render.getRelativeVector(self.controler.cameraNode, force)
            
            self.sphere_np.node().setActive(True)
            self.sphere_np.node().applyCentralForce(force)        
            self.sphere_np.node().applyTorque(torque)
        
    def update(self, task):
        if self.pause:
             self.sfx['hum'].setVolume(0.0)
        else:   
            dt = globalClock.getDt()        
            self.time-=dt            
            self.processInput(dt)              
            self.controler.cameraNode.setPos(self.sphere_np.getPos())
            
            max_v=Vec3(-10.0, -10.0, -10.0)
            min_v=Vec3(10.0, 10.0, 10.0)
            contacts=0
            for obj in self.bullet_segments:
                contacts+= self.world.contactTestPair(self.sphere_np.node(),obj.node()).getNumContacts()
            if  contacts>0:   
                if self.isFlying:                    
                    if self.turbo:
                        if not  self.sfx['contact_turbo'].status()== 2:
                            self.sfx['contact_turbo'].play()
                    else:
                        if not  self.sfx['contact'].status()== 2:
                            self.sfx['contact'].play()
                self.isFlying=False               
            else:
                self.isFlying=True              
            v=self.sphere_np.node().getAngularVelocity()
            new_v=v.fmax(max_v)
            new_v=new_v.fmin(min_v)       
            self.sphere_np.node().setAngularVelocity(Vec3(new_v)) 
            
            if self.world.contactTestPair(self.trigger1.node(),self.ghost.node()).getNumContacts()>0:
                self.generateFirewallColor(globalClock.getFrameTime())
                #print "trigger1!", globalClock.getFrameTime()                      
            if self.world.contactTestPair(self.trigger2.node(),self.ghost.node()).getNumContacts()>0:
                self.reGenerate(globalClock.getFrameTime())
                #print "trigger2!", globalClock.getFrameTime()
            for ghost in self.powerups_ghost:
                if ghost:
                    if ghost.node().getNumOverlappingNodes() > 0:
                        id =self.powerups_ghost.index(ghost)
                        self.world.removeGhost(ghost.node())
                        self.powerups_ghost[id]=None
                        self.powerups_vfx[id].remove_loop()
                        self.powerups_vfx[id]=None
                        self.powerups[id].removeNode()
                        self.powerups[id]=None
                        self.powerups_function[id]()                    
                        break  
            self.world.doPhysics(dt, 10, 1.0/360.0) 
            self.particle_np.setPos(self.sphere_np.getPos())
            speed=abs(self.sphere_np.node().getLinearVelocity().dot(Vec3(1.0, 1.0,1.0)))        
            volume= (speed/50.0)
            #print volume
            if abs(volume-self.last_volume)>0.05:
                LerpFunc(self.adjustHum,fromData=self.last_volume,toData=volume, duration=.2, blendType='easeOut').start()
                self.last_volume=volume 
            #self.axis_1.setH(self.axis_1.getH()+dt) 
            self.firewal_offset+=dt*0.3
            if self.firewal_offset>=1.0:
                self.firewal_offset=0
            self.firewall.setTexOffset(TextureStage.getDefault(), 0, -self.firewal_offset)
        return task.cont
    
    def lightFlick(self, task):
        r=0.005*random.randrange(1,20)
        self.pLight.setAttenuation(Point3(0, 0.5, r))
        return task.again
        
    def adjustHum(self, t):
        self.sfx['hum'].setPlayRate(t+1.0)
        self.sfx['hum'].setVolume(min(1.0,t))
        
    def doNothing(self):
        pass
        
    def loadSegment(self, file_name, base_offset, heading=0, powerup=None, function=None):
        root=self.axis.attachNewNode('segment')
        segment=loader.loadModel(file_name)
        segment.wrtReparentTo(root)  
        root.setH(heading) 
        root.setPos(base_offset)
        if powerup:      
            offset= random.choice([Vec3(12.0, 0.0,15.0),Vec3(0,0,-1.8),Vec3(5.0, 0.0,0.0),Vec3(-7.0, 0.0,0.0),Vec3(-11.0, 0.0,3.0),Vec3(-14.0, 0.0,7.0)])
            segment.setPos(segment,offset)            
            powerup_np=render.attachNewNode('powerup')            
            powerup_np.setPos(segment.getPos(render))            
            vfx_node=vfx(powerup_np, texture=powerup,scale=1.0,depthTest=True, depthWrite=True)
            vfx_node.loop()
            segment.hide()
            vfx_node.vfx.show()
            self.powerups.append(powerup_np)
            self.powerups_vfx.append(vfx_node) 
            ghost_shape = BulletSphereShape(0.4)
            ghost = self.world_np.attachNewNode(BulletGhostNode('Ghost'))
            ghost.node().addShape(ghost_shape)
            ghost.setPos(segment.getPos(render)) 
            ghost.setCollideMask(BitMask32.bit(3))
            self.world.attachGhost(ghost.node()) 
            self.powerups_ghost.append(ghost)  
            self.powerups_function.append(function)            
        else:
            geom = root.findAllMatches('**/+GeomNode').getPath(0).node().getGeom(0)        
            mesh = BulletTriangleMesh()
            mesh.addGeom(geom)
            shape = BulletTriangleMeshShape(mesh, dynamic=False)            
            bullet_segment = self.world_np.attachNewNode(BulletRigidBodyNode('Mesh'))
            bullet_segment.node().addShape(shape)
            mask=BitMask32()
            mask.setBit(1)
            mask.setBit(2)
            bullet_segment.setCollideMask(mask) 
            bullet_segment.setPos(segment.getPos(render)) 
            bullet_segment.setHpr(segment.getHpr(render))        
            self.world.attachRigidBody(bullet_segment.node())        
            self.segments.append(segment)        
            self.bullet_segments.append(bullet_segment)
            
    def generate(self, offset=Vec3(0,0,0), start_ang=0, ang=10, count=18):
        powerup_vfx=['vfx/powerup_turbo.png','vfx/powerup_turbo.png', 'vfx/powerup_cyan.png', 'vfx/powerup_green.png', 'vfx/powerup_mangana.png', 'vfx/powerup_red.png', 'vfx/powerup_data.png', 'vfx/powerup_data.png']
        powerup_func=[self.setTurbo, self.setTurbo, self.setCyan, self.setGreen, self.setMangana, self.setRed, self.setPickup, self.setPickup]
        for i in range(count):
            r=random.randint(1,24)
            self.loadSegment('models/ramp'+str(r), offset, start_ang+ang*i) 
            r=random.randint(0,7)
            self.loadSegment('models/powerup', offset, start_ang+ang*i, powerup_vfx[r], powerup_func[r]) 
            
    def setup(self):           
        self.world_np = render.attachNewNode('World')
        self.world = BulletWorld()
        self.world.setGravity(Vec3(0, 0, -9.81))
        
        self.debugNP = self.world_np.attachNewNode(BulletDebugNode('Debug'))
        #self.debugNP.show()
        self.world.setDebugNode(self.debugNP.node())
        
        self.segments=[]
        self.bullet_segments=[]
        self.powerups=[]   
        self.powerups_vfx=[]
        self.powerups_ghost=[]
        self.powerups_function=[]
        self.axis=render.attachNewNode('axis1')
        self.axis.setPos(-250, 0, 0)
                      
        #load the base level
        level=loader.loadModel('models/level')
        level.reparentTo(render)
        geom = level.findAllMatches('**/+GeomNode').getPath(0).node().getGeom(0)        
        mesh = BulletTriangleMesh()
        mesh.addGeom(geom)
        shape = BulletTriangleMeshShape(mesh, dynamic=False)            
        level_bullet = self.world_np.attachNewNode(BulletRigidBodyNode('Mesh'))
        level_bullet.node().addShape(shape)
        mask=BitMask32()
        mask.setBit(1)
        mask.setBit(2)
        level_bullet.setCollideMask(mask)         
        self.world.attachRigidBody(level_bullet.node())
        #load the firewal
        self.firewall=loader.loadModel('models/firewall')
        self.firewall.reparentTo(render)
        geom = self.firewall.findAllMatches('**/+GeomNode').getPath(0).node().getGeom(0)        
        mesh = BulletTriangleMesh()
        mesh.addGeom(geom)
        shape = BulletTriangleMeshShape(mesh, dynamic=False)            
        self.firewall_bullet= self.world_np.attachNewNode(BulletRigidBodyNode('firewall'))
        self.firewall_bullet.node().addShape(shape)
        self.firewall_bullet.setCollideMask(BitMask32.allOff())   
        self.world.attachRigidBody(self.firewall_bullet.node())
        self.firewall.setLightOff()
        self.firewall.setColor(.75, 1, 1, 1)
        self.firewal_offset=0
        
        #triggers
        trigger1=loader.loadModel('models/trigger1')        
        geom = trigger1.findAllMatches('**/+GeomNode').getPath(0).node().getGeom(0)        
        mesh = BulletTriangleMesh()
        mesh.addGeom(geom)
        shape = BulletTriangleMeshShape(mesh, dynamic=False)            
        self.trigger1= self.world_np.attachNewNode(BulletRigidBodyNode('triggerr'))
        self.trigger1.node().addShape(shape)
        mask=BitMask32()
        mask.setBit(2)
        self.trigger1.setCollideMask(mask)        
        self.world.attachRigidBody(self.trigger1.node())
        
        trigger2=loader.loadModel('models/trigger2')        
        geom = trigger2.findAllMatches('**/+GeomNode').getPath(0).node().getGeom(0)        
        mesh = BulletTriangleMesh()
        mesh.addGeom(geom)
        shape = BulletTriangleMeshShape(mesh, dynamic=False)            
        self.trigger2= self.world_np.attachNewNode(BulletRigidBodyNode('triggerr'))
        self.trigger2.node().addShape(shape)
        mask=BitMask32()
        mask.setBit(2)
        self.trigger2.setCollideMask(mask)        
        self.world.attachRigidBody(self.trigger2.node())
        
        
        #self.generate(self.axis_1)
        self.generate(offset=Vec3(0, 500, 0), start_ang=0, ang=10, count=27)
        self.generate(offset=Vec3(0,0, 0), start_ang=90, ang=-10, count=27)
        
        # Sphere-car
        shape = BulletSphereShape(1.0)
        self.sphere_np = self.world_np.attachNewNode(BulletRigidBodyNode('Car'))
        self.sphere_np.node().setMass(2.0)
        self.sphere_np.node().setFriction(1.0)
        self.sphere_np.node().addShape(shape)
        self.sphere_np.node().setAngularDamping(0.2)
        self.sphere_np.setPos(0, 500, 2)   
        mask=BitMask32()
        mask.setBit(1) #level
        # mask.setBit(2) #triggers
        mask.setBit(3) #powerups
        mask.setBit(4) #firewall cyan
        #mask.setBit(5) #firewall Green
        #mask.setBit(6) #firewall Mangana
        #mask.setBit(7) #firewall Red             
        self.sphere_np.setCollideMask(mask)
        self.world.attachRigidBody(self.sphere_np.node())
        #visual part
        visualNP = loader.loadModel('models/sphere.egg')
        visualNP.setScale(1.0/6.0)
        visualNP.reparentTo(self.sphere_np)
        self.tron_part=visualNP.find("**/tron_part")
        self.tron_part.setColor(.75, 1, 1, 1)
        self.tron_part.setLightOff()
        #print visualNP.ls()
        #ghost part
        ghost_shape = BulletSphereShape(1.0)
        self.ghost = self.sphere_np.attachNewNode(BulletGhostNode('Ghost'))
        self.ghost.node().addShape(ghost_shape)
        self.ghost.setCollideMask(BitMask32.bit(2))
        self.world.attachGhost(self.ghost.node())  
        #vfx part
        self.electric_vfx=vfx(self.sphere_np, texture='vfx/electric1.png',scale=1.0)
        self.electric_vfx.loop(speed=0.01)
        self.electric_vfx.vfx.setColor(.75, 1, 1, 1)
        self.electric_vfx.vfx.hide()
        self.particle_np  = render.attachNewNode("particleNode")
        #self.particle_np.setLightOff()
        self.p = ParticleEffect()
        self.p.loadConfig('vfx/motion2.ptf')
        for geom in self.p.findAllMatches('**/+GeomNode'):
            geom.setLightOff()
        self.p.start(parent=self.particle_np, renderParent = render)
        #light part
        self.pLight = PointLight('plight')
        self.pLight.setColor(VBase4(.75, 1, 1, 1))
        self.pLight.setAttenuation(Point3(1, 0.00, 0.2))         
        pLightNode = self.sphere_np.attachNewNode(self.pLight)
        render.setLight(pLightNode)    
        
        self.pLight2 = PointLight('plight2')
        self.pLight2.setColor(VBase4(0.5, 0.5,0.5, 1))
        self.pLight2.setAttenuation(Point3(0.5, 0.002, 0))         
        pLightNode2 = self.particle_np.attachNewNode(self.pLight2)
        pLightNode2.setPos(0,0, 9)
        render.setLight(pLightNode2)
        #taskMgr.doMethodLater(0.05, self.lightFlick,'light_task')
        
        #sounds
        self.sfx={}
        self.sfx['contact_turbo']=base.loader.loadSfx("sfx/contact.ogg")
        self.sfx['contact']=base.loader.loadSfx("sfx/contact2.ogg")
        self.sfx['hum']=base.loader.loadSfx("sfx/dynamohum.ogg")
        self.sfx['pickup1']=base.loader.loadSfx("sfx/pickup_1.ogg")
        self.sfx['pickup2']=base.loader.loadSfx("sfx/pickup_2.ogg")
        self.sfx['pickup3']=base.loader.loadSfx("sfx/pickup_3.ogg")
        self.sfx['pickup4']=base.loader.loadSfx("sfx/pickup_4.ogg")
        self.sfx['pickup5']=base.loader.loadSfx("sfx/pickup_5.ogg")
        self.sfx['pickup_turbo']=base.loader.loadSfx("sfx/pickup_turbo.ogg")
        self.sfx['hum'].setLoop(True)
        self.sfx['hum'].play()
        
        #music
        self.music=base.loadMusic('music/tactical_pursuit.ogg')
        base.musicManager.setVolume(0.5)
        self.music.setLoop(True)
        self.music.play()
        
game = Game()
base.run()

